//---------------------------------------------------------------------------
// Copyright (C) 2001 Dallas Semiconductor Corporation, All Rights Reserved.
// 
// Permission is hereby granted, free of charge, to any person obtaining a 
// copy of this software and associated documentation files (the "Software"), 
// to deal in the Software without restriction, including without limitation 
// the rights to use, copy, modify, merge, publish, distribute, sublicense, 
// and/or sell copies of the Software, and to permit persons to whom the 
// Software is furnished to do so, subject to the following conditions:
// 
// The above copyright notice and this permission notice shall be included 
// in all copies or substantial portions of the Software.
// 
// THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS 
// OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF 
// MERCHANTABILITY,  FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. 
// IN NO EVENT SHALL DALLAS SEMICONDUCTOR BE LIABLE FOR ANY CLAIM, DAMAGES 
// OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, 
// ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR 
// OTHER DEALINGS IN THE SOFTWARE.
// 
// Except as contained in this notice, the name of Dallas Semiconductor 
// shall not be used except as stated in the Dallas Semiconductor 
// Branding Policy. 
//---------------------------------------------------------------------------
//
//  getV.c - This utility gets the Volts for pins Vad and Vdd from the DS2438. 
//
//  Version:   2.00
//  History: 

#include <windows.h>
#include "ibtmexcw.h"

// defines
#define SBATTERY_FAM  0x26

// Local Function Prototypes 
short FindFirstFamily(short);
void PrintHeader(char *);
short Aquire1WireNet(void);
void Release1WireNet(void);
float Volt_Reading(int);
double Get_Temperature();
void Exit_Prog(char *, short);
void setcrc8(uchar);
uchar docrc8(uchar);

// Global Variables to hold TMEX state 
short PortType, PortNum;
long session_handle;                         
uchar state_buffer[5120];
uchar utilcrc8;
static uchar dscrc_table[] = {
        0, 94,188,226, 97, 63,221,131,194,156,126, 32,163,253, 31, 65,
      157,195, 33,127,252,162, 64, 30, 95,  1,227,189, 62, 96,130,220,
       35,125,159,193, 66, 28,254,160,225,191, 93,  3,128,222, 60, 98,
      190,224,  2, 92,223,129, 99, 61,124, 34,192,158, 29, 67,161,255,
       70, 24,250,164, 39,121,155,197,132,218, 56,102,229,187, 89,  7,
      219,133,103, 57,186,228,  6, 88, 25, 71,165,251,120, 38,196,154,
      101, 59,217,135,  4, 90,184,230,167,249, 27, 69,198,152,122, 36,
      248,166, 68, 26,153,199, 37,123, 58,100,134,216, 91,  5,231,185,
      140,210, 48,110,237,179, 81, 15, 78, 16,242,172, 47,113,147,205,
       17, 79,173,243,112, 46,204,146,211,141,111, 49,178,236, 14, 80,
      175,241, 19, 77,206,144,114, 44,109, 51,209,143, 12, 82,176,238,
       50,108,142,208, 83, 13,239,177,240,174, 76, 18,145,207, 45,115,
      202,148,118, 40,171,245, 23, 73,  8, 86,180,234,105, 55,213,139,
       87,  9,235,181, 54,104,138,212,149,203, 41,119,244,170, 72, 22,
      233,183, 85, 11,136,214, 52,106, 43,117,151,201, 74, 20,246,168,
      116, 42,200,150, 21, 75,169,247,182,232, 10, 84,215,137,107, 53};


//----------------------------------------------------------------------
//  This is the Main routine for debit
//
int main(short argc, char **argv)
{
   short found = FALSE;
   float Vdd,Vad;
   double humid,temp;
   char timebuff[128];

   // check arguments 
   if (argc > 1)
   {
       printf("\nusage: tmhumid\n"
              "  - read the Vad, Vdd, Temperature and Calculate Humidity for DS2438.\n"
              "  - version 1.00\n");
       Exit_Prog("",BAD_ARGUMENT);
   }

   // read the default PortNum and PortType 
   if (!TMReadDefaultPort(&PortNum,&PortType))
   {
       printf("ERROR, Could not read the default PortNum and PortType!\n");
       exit(1);
   }

   PrintHeader("tmhumid Version 1.00");

   // find the device
   found = FALSE;
   if(Aquire1WireNet())
   {
      // setup the MicroLan 
      if (TMSetup(session_handle) == 1)
      {
         printf("Searching for a DS2438..\n\n");

         // look for the first switch iButton (family type 0x26) 
         if (FindFirstFamily(SBATTERY_FAM))
            found = TRUE;
         // iButton not on MicroLan 
         else   
            printf("DS2438 not found on MicroLan!"); 
      }   
      // MicroLan port not valid 
      else   
         printf("MicroLan setup failed!"); 
      // release port 
      Release1WireNet();
   }

   if(found)
   {
      printf("  date    Vdd   Vad    temp  humidity\n");
      do
      {

         Vdd = Volt_Reading(TRUE);
         Vad = Volt_Reading(FALSE);

         temp = Get_Temperature();

         humid = (((Vad/Vdd) - 0.16)/0.0062)/(1.0546 - 0.00216 * temp);
        
         _tzset();
         _strtime(timebuff);

         printf("%s, %4.2f, %4.2f, %4.2fC, %4.2f%%\n",timebuff,Vdd,Vad,temp,humid);
 
      }while(!kbhit());  
   }

   return 1;
}

//----------------------------------------------------------------------
// Find the first device with a particular family
// 
short FindFirstFamily(short family)
{
   short j;
   short ROM[9];
   
   // set up to find the first device with family 'family 
   if (TMFamilySearchSetup(session_handle,&state_buffer,family) == 1)
   { 
      // get first device in list with the specified family 
      if (TMNext(session_handle,state_buffer) == 1)
      {
         // read the rom number 
         ROM[0] = 0;
         TMRom(session_handle,state_buffer,ROM);   
         
         // check if correct type 
         if ((family & 0x7F) == (ROM[0] & 0x7F))
         {
            printf("1-Wire Network Address: ");  
            for (j = 7; j >= 0; j--)
               printf("%02X",ROM[j]);
            printf("\n\n");
            return 1;
         }        
      }
   }
     
   // failed to find device of that type 
   return 0;                              
}

//----------------------------------------------------------------------
// Aquire a session on the 1-Wire Net
// return TRUE(1)  : session on 1-Wire Net aquired
//        FALSE(0) : could not aquire a session on the 1-Wire Net
//                                                          
short Aquire1WireNet(void)
{
   short printmsg=1;
   ulong tm;

   // set timeout
   tm = GetTickCount() + 2000;

   // loop to get a session handle or timeout 
   do
   {
      // attempt to get a session 
      session_handle = TMExtendedStartSession(PortNum,PortType,NULL);
      if (session_handle > 0)  
         return TRUE;
      else if (session_handle == 0)
      {
         if (printmsg)
         {
            printf("Waiting to get access to 1-Wire Net\n");
            printmsg = 0;
         }
      }
      else if (session_handle < 0)   
      {
         printf("No Hardware Driver for this type found!\n");
         return FALSE;
      }
   }
   while (tm > GetTickCount());

   return FALSE;   
}

//----------------------------------------------------------------------
// Give up the session on the 1-Wire Net
//                                                          
void Release1WireNet(void)
{
   // end the session if one is open
   TMEndSession(session_handle);
}

//----------------------------------------------------------------------
// To set the status/configuration register to convert the Vdd or the Vad
// data.  Returns if it was successful or not.
// 
int Volt_AD(int vdd)
{
   uchar send_block[50];
   uchar test;
   short send_cnt=0;
   int i;
   ushort lastcrc8;
   int busybyte; 

   if(Aquire1WireNet())
   {
      if(TMAccess(session_handle,state_buffer) > 0)
      {
         // Recall the Status/Configuration page
         // Recall command
         send_block[send_cnt++] = 0xB8;

         // Page to Recall
         send_block[send_cnt++] = 0x00;

         if(TMBlockStream(session_handle,send_block,send_cnt) != send_cnt)
         {
            Release1WireNet();
            return FALSE;
         }

         send_cnt = 0;
      }

      if(TMAccess(session_handle,state_buffer) > 0)
      {
         // Read the Status/Configuration byte
         // Read scratchpad command
         send_block[send_cnt++] = 0xBE;

         // Page for the Status/Configuration byte
         send_block[send_cnt++] = 0x00;

         for(i=0;i<9;i++)
            send_block[send_cnt++] = 0xFF;
            
         if(TMBlockStream(session_handle,send_block,send_cnt) == send_cnt)
         {
            setcrc8(0);

            for(i=2;i<send_cnt;i++)
               lastcrc8 = docrc8(send_block[i]);

            if(lastcrc8 != 0x00)
            {
               Release1WireNet();
               return FALSE;
            }
         }//Block
         else
         {
            Release1WireNet();
            return FALSE;
         }

         test = send_block[2] & 0x08;
         if(((test == 0x08) && vdd) || ((test == 0x00) && !(vdd)))
         {
            Release1WireNet();
            return TRUE;
         }
      }//Access

      if(TMAccess(session_handle,state_buffer) > 0)
      {
         send_cnt = 0;
         // Write the Status/Configuration byte
         // Write scratchpad command
         send_block[send_cnt++] = 0x4E;

         // Write page
         send_block[send_cnt++] = 0x00;

         if(vdd)
            send_block[send_cnt++] = send_block[2] | 0x08;
         else
            send_block[send_cnt++] = send_block[2] & 0xF7;

         for(i=0;i<7;i++)
            send_block[send_cnt++] = send_block[i+4];

         if(TMBlockStream(session_handle,send_block,send_cnt) == send_cnt)
         {
            send_cnt = 0;

            if(TMAccess(session_handle,state_buffer) > 0)
            {
               // Copy the Status/Configuration byte
               // Copy scratchpad command
               send_block[send_cnt++] = 0x48;

               // Copy page
               send_block[send_cnt++] = 0x00;

               if(TMBlockStream(session_handle,send_block,send_cnt) == send_cnt)
               {
                  busybyte = TMTouchByte(session_handle,0xFF);
         
                  while(busybyte == 0)
                     busybyte = TMTouchByte(session_handle,0xFF);

                  Release1WireNet();

                  return TRUE;
               }//Block
            }//Access
         }//Block

      }//Access

   }//Acquire

   Release1WireNet();
   return FALSE;
}
      
//----------------------------------------------------------------------
// Get the Vad or the Vdd reading depending on the boolean passed in.  
// Returns a float value for the volts in reading Vad or Vdd.
//  
float Volt_Reading(int vdd)
{
   uchar send_block[50];
   short send_cnt=0;
   int i;
   int busybyte; 
   ushort lastcrc8;
   ushort volts;
   float ret=-1.0;

   if(Volt_AD(vdd))
   {
      if(Aquire1WireNet())
      {

         if(TMAccess(session_handle,state_buffer) > 0)
         {
            if(!TMTouchByte(session_handle,0xB4))
            {
               printf("DIDN'T WRITE CORRECTLY\n");
               Release1WireNet();
               return ret;
            }

            busybyte = TMTouchByte(session_handle,0xFF);
         
            while(busybyte == 0)
               busybyte = TMTouchByte(session_handle,0xFF);
         }

         if(TMAccess(session_handle,state_buffer) > 0)
         {
            // Recall the Status/Configuration page
            // Recall command
            send_block[send_cnt++] = 0xB8;

            // Page to Recall
            send_block[send_cnt++] = 0x00;

            if(TMBlockStream(session_handle,send_block,send_cnt) != send_cnt)
            {
               Release1WireNet();
               return ret;
            }
         }


         send_cnt = 0;

         if(TMAccess(session_handle,state_buffer) > 0)
         {
            // Read the Status/Configuration byte
            // Read scratchpad command
            send_block[send_cnt++] = 0xBE;

            // Page for the Status/Configuration byte
            send_block[send_cnt++] = 0x00;

            for(i=0;i<9;i++)
               send_block[send_cnt++] = 0xFF;

            if(TMBlockStream(session_handle,send_block,send_cnt) == send_cnt)
            {
               setcrc8(0);

               for(i=2;i<send_cnt;i++)
                  lastcrc8 = docrc8(send_block[i]);

               if(lastcrc8 != 0x00)
               {
                  Release1WireNet();
                  return ret;
               }

            }
            else
            {
               Release1WireNet();
               return ret;  
            }
      
            volts = (send_block[6] << 8) | send_block[5];
            ret = (float) volts/100;
         }//Access

         Release1WireNet();
      }

   }

   return ret;

}

//----------------------------------------------------------------------
// Returns the temperature of type double in C.
// 
double Get_Temperature()
{
   double ret=-1.0;
   uchar send_block[50];
   short send_cnt=0;
   int i;
   ushort lastcrc8;

   if(Aquire1WireNet())
   {
      if(TMAccess(session_handle,state_buffer) > 0)
         // Convert Temperature command
         TMTouchByte(session_handle,0x44);
      else
         printf("ACCESS ONE DIDNT WORK.\n");

      Sleep(10);

      if(TMAccess(session_handle,state_buffer) > 0)
      {
         // Read the Status/Configuration byte
         // Read scratchpad command
         send_block[send_cnt++] = 0xBE;

         // Page for the Status/Configuration byte
         send_block[send_cnt++] = 0x00;

         for(i=0;i<9;i++)
            send_block[send_cnt++] = 0xFF;

         if(TMBlockStream(session_handle,send_block,send_cnt) == send_cnt)
         {
            setcrc8(0);

            for(i=2;i<send_cnt;i++)
               lastcrc8 = docrc8(send_block[i]);

            if(lastcrc8 != 0x00)
            {
               Release1WireNet();
               printf("FIRST CRC\n");
               return ret;
            }

         }
         else
         {
            Release1WireNet();
            printf("FIRST BLOCK STREAM DIDNT WORK.\n");
            return ret;  
         }
      
         ret = (((send_block[4] << 8) | send_block[3]) >> 3) * 0.03125;
      }//Access
      else
         printf("ACCESS DIDN'T WORK.\n");

      Release1WireNet();
   }

   return ret;
}

//--------------------------------------------------------------------------
//  Prints a message, and the current driver versions.
//
void PrintHeader(char *msg)
{
   char tstr[80];       

   // print a header with driver version info   
   printf("\n%s\n"
         "Dallas Semiconductor Corporation\n"
         "Copyright (C) 1992-2001\n\n",msg);
   printf("Port number: %d    Port type: %d\n",PortNum,PortType);
   Get_Version(tstr);
   printf("Main driver version: %s\n",tstr);
   printf("Sub-driver version, Type%d:",PortType);
   if (TMGetTypeVersion(PortType,tstr) < 0)
   {
      printf("No Hardware Driver for this type found!");
      exit(1);
   }
   printf(" %s\n\n\n",tstr);
}

//--------------------------------------------------------------------------
// Reset crc8 to the value passed in
//
// 'reset'    - data to set crc8 to
//
void setcrc8(uchar reset)
{
   utilcrc8 = reset;
   return;
}

//--------------------------------------------------------------------------
// Update the Dallas Semiconductor One Wire CRC (utilcrc8) from the global
// variable utilcrc8 and the argument.  
//
// 'x'        - data byte to calculate the 8 bit crc from
//
// Returns: the updated utilcrc8.
//
uchar docrc8(uchar x)
{
   utilcrc8 = dscrc_table[utilcrc8 ^ x];
   return utilcrc8;
}

//--------------------------------------------------------------------------
//  Prints a message, beeps and exits the program.
// 
void Exit_Prog(char *msg, short errcd)
{
     // print the message left justified 
     printf("\r%s",msg);

     //  error beep if return code is not 0 
     if (errcd != NORMAL_EXIT)
        MessageBeep(MB_ICONEXCLAMATION);

     //  return error code passed 
     exit(errcd);
}